from django.contrib import admin
from django.contrib.auth.admin import UserAdmin
from testapp.models import CustomUser, UserAuditTrail

admin.site.register(UserAuditTrail, admin.ModelAdmin)
admin.site.register(CustomUser, UserAdmin)



# Generated by Django 4.2.4 on 2023-09-02 06:29

import django.contrib.auth.models
import django.contrib.auth.validators
import django.contrib.postgres.fields.hstore
from django.db import migrations, models
import django.utils.timezone
import pgtrigger.compiler
import pgtrigger.migrations
import testapp.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('auth', '0012_alter_user_first_name_max_length'),
    ]

    operations = [
        migrations.CreateModel(
            name='UserAuditTrail',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('before', django.contrib.postgres.fields.hstore.HStoreField()),
                ('after', django.contrib.postgres.fields.hstore.HStoreField()),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='CustomUser',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('password', models.CharField(max_length=128, verbose_name='password')),
                ('last_login', models.DateTimeField(blank=True, null=True, verbose_name='last login')),
                ('is_superuser', models.BooleanField(default=False, help_text='Designates that this user has all permissions without explicitly assigning them.', verbose_name='superuser status')),
                ('username', models.CharField(error_messages={'unique': 'A user with that username already exists.'}, help_text='Required. 150 characters or fewer. Letters, digits and @/./+/-/_ only.', max_length=150, unique=True, validators=[django.contrib.auth.validators.UnicodeUsernameValidator()], verbose_name='username')),
                ('first_name', models.CharField(blank=True, max_length=150, verbose_name='first name')),
                ('last_name', models.CharField(blank=True, max_length=150, verbose_name='last name')),
                ('email', models.EmailField(blank=True, max_length=254, verbose_name='email address')),
                ('is_staff', models.BooleanField(default=False, help_text='Designates whether the user can log into this admin site.', verbose_name='staff status')),
                ('is_active', models.BooleanField(default=True, help_text='Designates whether this user should be treated as active. Unselect this instead of deleting accounts.', verbose_name='active')),
                ('date_joined', models.DateTimeField(default=django.utils.timezone.now, verbose_name='date joined')),
                ('groups', models.ManyToManyField(blank=True, help_text='The groups this user belongs to. A user will get all permissions granted to each of their groups.', related_name='user_set', related_query_name='user', to='auth.group', verbose_name='groups')),
                ('user_permissions', models.ManyToManyField(blank=True, help_text='Specific permissions for this user.', related_name='user_set', related_query_name='user', to='auth.permission', verbose_name='user permissions')),
            ],
            options={
                'abstract': False,
                'audit_table': testapp.models.UserAuditTrail,
            },
            managers=[
                ('objects', django.contrib.auth.models.UserManager()),
            ],
        ),
        pgtrigger.migrations.AddTrigger(
            model_name='customuser',
            trigger=pgtrigger.compiler.Trigger(name='track_history_create', sql=pgtrigger.compiler.UpsertTriggerSql(func='\n    INSERT INTO testapp_useraudittrail(before, after)\n        SELECT hstore(old), hstore(new);\n    RETURN new;\n', hash='ae8638ede3eeabe979e5e9f0c6a22a11874cc9dd', operation='INSERT', pgid='pgtrigger_track_history_create_54cee', table='testapp_customuser', when='AFTER')),
        ),
        pgtrigger.migrations.AddTrigger(
            model_name='customuser',
            trigger=pgtrigger.compiler.Trigger(name='track_history_update', sql=pgtrigger.compiler.UpsertTriggerSql(func='\n    INSERT INTO testapp_useraudittrail(before, after)\n        SELECT hstore(old), hstore(new);\n    RETURN new;\n', hash='741990c94a8594b73f00ca2f222b6953a7064671', operation='UPDATE', pgid='pgtrigger_track_history_update_f7583', table='testapp_customuser', when='AFTER')),
        ),
        pgtrigger.migrations.AddTrigger(
            model_name='customuser',
            trigger=pgtrigger.compiler.Trigger(name='track_history_delete', sql=pgtrigger.compiler.UpsertTriggerSql(func='\n    INSERT INTO testapp_useraudittrail(before, after)\n        SELECT hstore(old), hstore(new);\n    RETURN new;\n', hash='478b6e2629b988ab0743fe725acdc395d923b2dd', operation='DELETE', pgid='pgtrigger_track_history_delete_5ef81', table='testapp_customuser', when='AFTER')),
        ),
    ]



from django.db import models
from django.contrib.postgres.fields import HStoreField
import pgtrigger


class AuditTrail(models.Model):
    """
    An audit trail model mixin, no fields except for a timestamp
    perhaps should be added to tables that utilize this.

    If this data will be accessed frequently consider an IndexedAuditTrail instead.
    """
    before = HStoreField()
    after = HStoreField()

    class Meta:
        abstract = True


class IndexedAuditTrail(models.Model):
    """
    An indexed audit trail model mixin, no fields except for a timestamp
    perhaps should be added to tables that utilize this.
    """
    before = HStoreField(db_index=True)
    after = HStoreField(db_index=True)

    class Meta:
        abstract = True


TRIGGER_SQL = """
    INSERT INTO {meta.audit_table._meta.db_table}(before, after)
        SELECT hstore(old), hstore(new);
    RETURN new;
"""


class AuditTrigger(models.Model):

    class Meta:
        abstract = True
        triggers = [
            pgtrigger.Trigger(
                name="track_history_create",
                level=pgtrigger.Row,
                when=pgtrigger.After,
                operation=pgtrigger.Insert,
                func=pgtrigger.Func(TRIGGER_SQL)
            ),
            pgtrigger.Trigger(
                name="track_history_update",
                level=pgtrigger.Row,
                when=pgtrigger.After,
                operation=pgtrigger.Update,
                func=pgtrigger.Func(TRIGGER_SQL)
            ),
            pgtrigger.Trigger(
                name="track_history_delete",
                level=pgtrigger.Row,
                when=pgtrigger.After,
                operation=pgtrigger.Delete,
                func=pgtrigger.Func(TRIGGER_SQL)
            )
        ]



# Generated by Django 4.2.4 on 2023-09-02 06:01
from django.contrib.postgres.operations import HStoreExtension
from django.db import migrations


class Migration(migrations.Migration):
    initial = True

    dependencies = [
    ]

    operations = [
        HStoreExtension()
    ]



from django.apps import AppConfig

from django.db.migrations import state
from django.db.models import options


if "triggers" not in state.DEFAULT_NAMES:  # pragma: no branch
    state.DEFAULT_NAMES = tuple(state.DEFAULT_NAMES) + ("triggers",)
if "triggers" not in options.DEFAULT_NAMES:  # pragma: no branch
    options.DEFAULT_NAMES = tuple(options.DEFAULT_NAMES) + ("triggers",)

if "audit_table" not in options.DEFAULT_NAMES:  # pragma: no branch
    options.DEFAULT_NAMES = tuple(options.DEFAULT_NAMES) + ("audit_table", "audit_options",)
if "audit_table" not in state.DEFAULT_NAMES:  # pragma: no branch
    state.DEFAULT_NAMES = tuple(state.DEFAULT_NAMES) + ("audit_table", "audit_options",)


def patch_migrations():
    """
    Patch the autodetector and model state detection if migrations are turned on
    """
    if "triggers" not in state.DEFAULT_NAMES:  # pragma: no branch
        state.DEFAULT_NAMES = tuple(state.DEFAULT_NAMES) + ("triggers",)

    if "audit_table" not in state.DEFAULT_NAMES:  # pragma: no branch
        state.DEFAULT_NAMES = tuple(state.DEFAULT_NAMES) + ("audit_table", "audit_options",)

    if "triggers" not in options.DEFAULT_NAMES:  # pragma: no branch
        options.DEFAULT_NAMES = tuple(options.DEFAULT_NAMES) + ("triggers",)
    if "audit_table" not in options.DEFAULT_NAMES:  # pragma: no branch
        options.DEFAULT_NAMES = tuple(options.DEFAULT_NAMES) + ("audit_table", "audit_options",)


class DjangoAudimaticConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'django_audimatic'

    def ready(self):
        """
        Do all necessary patching
        """
        patch_migrations()



from django.contrib.auth.models import AbstractUser
from django_audimatic.models import AuditTrigger, AuditTrail


class UserAuditTrail(AuditTrail):
    pass


class CustomUser(AbstractUser, AuditTrigger):

    class Meta(AuditTrigger.Meta):
        audit_table = UserAuditTrail
